@@ -6,8 +6,7 @@
 ;; Maintainer: Chen Bin <chenbin.sh@gmail.com>
 ;; Keywords: mime, mail, email, html
 ;; Homepage: http://github.com/org-mime/org-mime
-;; Package-Version: 0.3.4
-;; Package-Revision: 90aa9081ec00
+;; Version: 0.3.4
 ;; Package-Requires: ((emacs "27.1"))
 
 ;; This file is not part of GNU Emacs.
@@ -141,21 +140,48 @@
   :group 'org-mime
   :type 'boolean)
 
+(defcustom org-mime-beautify-quoted-mail-style "margin:0 0 0 .8ex;border-left:1px #ccc solid;padding-left:1ex"
+  "Style for beautifying quoted mail."
+  :type 'string
+  :group 'org-mime)
+
 (defcustom org-mime-use-property-inheritance nil
   "Non-nil means al MAIL_ properties apply also for sub-levels."
   :group 'org-mime
   :type 'boolean)
 
 (defcustom org-mime-library 'mml
-  "Library to use for marking up MIME elements."
+  "Library used by org-mime for generating MIME markup.
+The value should be one of the following symbols:
+- `mml' – Use the Message/MML library (the default).
+- `semi' – Use SEMI (a MIME framework used by Wanderlust and older mail
+  clients).  Choose this if your mail setup relies on SEMI.
+- `vm' – Use VM's (ViewMail) internal MIME composition functions.
+  Select this if you prepare mail using VM.
+
+In most cases, `mml' is the correct choice."
   :group 'org-mime
-  :type '(choice 'mml 'semi 'vm))
+  :type '(choice
+          (const :tag "MML (message-mode, default)" mml)
+          (const :tag "SEMI (Wanderlust / SEMI-based MUAs)" semi)
+          (const :tag "VM (VM mail client)" vm)))
 
 (defcustom org-mime-export-ascii nil
-  "ASCII export options for text/plain.
-Default (nil) selects the original org file."
+  "Charset to use when exporting the text/plain part as ASCII.
+When nil (the default), org-mime inserts the original Org source text
+without running ASCII export.
+
+When non-nil, the value must be one of the symbols accepted by
+`org-ascii-charset':
+- `ascii'  – Use only plain, 7-bit ASCII characters.
+- `latin1' – Allow Latin-1 (ISO-8859-1) characters.
+- `utf-8'  – Allow full UTF-8 characters."
   :group 'org-mime
-  :type '(choice 'ascii 'latin1 'utf-8))
+  :type '(choice
+          (const :tag "Insert original Org source (no ASCII export)" nil)
+          (const :tag "ASCII (7-bit)" ascii)
+          (const :tag "Latin-1" latin1)
+          (const :tag "UTF-8" utf-8)))
 
 (defcustom org-mime-preserve-breaks t
   "Temporary value of `org-export-preserve-breaks' during mime encoding."
@@ -184,6 +210,35 @@ Default (nil) selects the original org file."
   :group 'org-mime
   :type 'string)
 
+(defcustom org-mime-instructions-hint "## org-mime hint: Press C-c C-c to commit change.\n"
+  "Instructions inserted in the org-mime org-mode buffer.
+The org-mime org-mode buffer is an org-mode buffer created from calling
+`org-mime-edit-mail-in-org-mode', used to edit the mail body in
+org-mode.
+
+The value of this option is the string inserted at the top of an
+org-mime org-mode buffer, meant to provide to the user instructions
+about how to \"commit\" the changes made in this buffer to the original,
+parent email composition buffer.
+
+The value of this option can also be nil, meaning no instructions hint
+is inserted into the org-mime org-mode buffer."
+  :group 'org-mime
+  :type '(choice (string :tag "Instructions hint")
+                 (const :tag "No hint" nil)))
+
+(defcustom org-mime-obey-display-buffer-p nil
+  "Whether `display-buffer-alist' is obeyed.
+Controls how windows are rearranged when calling
+`org-mime-edit-mail-in-org-mode'.
+
+The default (nil) means to delete all other windows then use
+`switch-to-buffer-other-window' to display the `org-mime-src-mode' org
+buffer.  When non-nil, instead, `display-buffer' is used, meaning the
+buffer will be displayed according to `display-buffer-alist'."
+  :group 'org-mime
+  :type 'boolean)
+
 (defvar org-mime-export-options '(:with-latex imagemagick)
   "Default export options which may override org buffer/subtree options.
 You could avoid exporting section-number/author/toc.
@@ -209,11 +264,17 @@ buffer holding the text to be exported.")
   "Enable debug logger.")
 
 ;; internal variables
-(defvar org-mime-src--overlay nil)
-(defvar org-mime-src--beg-marker nil)
-(defvar org-mime-src--end-marker nil)
-(defvar org-mime--saved-temp-window-config nil)
-(defconst org-mime-src--hint "## org-mime hint: Press C-c C-c to commit change.\n")
+(defvar-local org-mime-src--overlay nil)
+(put 'org-mime-src--overlay 'permanent-local t)
+
+(defvar-local org-mime-src--beg-marker nil)
+(put 'org-mime-src--beg-marker 'permanent-local t)
+
+(defvar-local org-mime-src--end-marker nil)
+(put 'org-mime-src--end-marker 'permanent-local t)
+
+(defvar-local org-mime--saved-temp-window-config nil)
+(put 'org-mime--saved-temp-window-config 'permanent-local t)
 
 (defun org-mime-get-buffer-export-options ()
   "Get export options in buffer."
@@ -314,7 +375,9 @@ HTML is the body of the message."
   (let ((quote-depth 0)
         (line-depth 0)
         (in-quote-p nil)
-        (quote-opening "<blockquote class=\"gmail_quote\" style=\"margin:0 0 0 .8ex;border-left:1px #ccc solid;padding-left:1ex\">\n\n<div>")
+        (quote-opening (concat "<blockquote class=\"gmail_quote\" style=\""
+                               org-mime-beautify-quoted-mail-style
+                               "\">\n\n<div>"))
         (quote-closing "\n</div></blockquote>\n"))
     (with-temp-buffer
       ;; clean title of quoted
@@ -497,10 +560,7 @@ CURRENT-FILE is used to calculate full path of images."
       (mapc (lambda (f)
               (when org-mime-debug (message "attaching: %s" f))
               (mml-attach-file f))
-            files))
-
-    ;; spacer
-    (insert "\n\n")))
+            files))))
 
 (defun org-mime-mail-body-begin ()
   "Get begin of mail body."
@@ -563,7 +623,7 @@ CURRENT-FILE is used to calculate full path of images."
 (defun org-mime-htmlize ()
   "Export a portion of an email to html using `org-mode'.
 If called with an active region only export that region, otherwise entire body."
-  (interactive)
+  (interactive nil message-mode)
   (when org-mime-debug (message "org-mime-htmlize called"))
 
   (let* ((region-p (org-region-active-p))
@@ -591,17 +651,14 @@ If called with an active region only export that region, otherwise entire body."
 
     ;; restore secure tags
     (when secure-tags
-      (insert (mapconcat #'identity secure-tags "\n"))
-      ;; spacer
-      (insert "\n\n"))
+      (insert (mapconcat #'identity secure-tags "\n")))
 
     ;; insert converted html
     (org-mime-insert-html-content plain file html)
 
     ;; restore part tags (attachments)
     (when part-tags
-      (insert (mapconcat #'identity part-tags "\n"))
-      (insert "\n\n"))))
+      (insert (mapconcat #'identity part-tags "\n")))))
 
 (defun org-mime--get-buffer-title ()
   "Get buffer title."
@@ -668,7 +725,7 @@ The following file keywords can be used to control the headers:
 #+MAIL_IN_REPLY_TO: <MESSAGE_ID>
 
 The cursor ends in the TO field."
-  (interactive)
+  (interactive nil org-mode)
   (run-hooks 'org-mime-send-buffer-hook)
   (let* ((org-html-klipsify-src nil)
          (file (buffer-file-name (current-buffer)))
@@ -713,7 +770,7 @@ Following headline properties can determine the mail headers.
   :MAIL_BCC: person3@gmail.com
   :MAIL_FROM: sender@gmail.com
   :END:"
-  (interactive "P")
+  (interactive "P" org-mode)
   (save-excursion
     (org-back-to-heading)
 
@@ -762,13 +819,13 @@ Following headline properties can determine the mail headers.
   "Get edited code."
   (save-excursion
     (goto-char (point-min))
-    (search-forward org-mime-src--hint (point-max) t)
+    (search-forward org-mime-instructions-hint (point-max) t)
     (goto-char (line-beginning-position))
     (buffer-substring-no-properties (point) (point-max))))
 
 (defun org-mime-edit-src-save ()
   "Save parent buffer with current state source-code buffer."
-  (interactive)
+  (interactive nil org-mime-src-mode)
   (set-buffer-modified-p nil)
   (let* ((edited-code (org-mime-edited-code))
          (beg org-mime-src--beg-marker)
@@ -801,16 +858,14 @@ Following headline properties can determine the mail headers.
 
 (defun org-mime-edit-src-exit ()
   "Kill current sub-editing buffer and return to source buffer."
-  (interactive)
+  (interactive nil org-mime-src-mode)
   (let* ((beg org-mime-src--beg-marker)
          (end org-mime-src--end-marker)
          (edit-buffer (current-buffer))
          (source-buffer (marker-buffer beg)))
     (org-mime-edit-src-save)
     (unless source-buffer (error "Source buffer disappeared.  Aborting"))
-    ;; Insert modified code.  Ensure it ends with a newline character.
-    (kill-buffer edit-buffer)
-
+    
     ;; to the beginning of the block opening line.
     (goto-char beg)
 
@@ -818,8 +873,13 @@ Following headline properties can determine the mail headers.
     (set-marker beg nil)
     (set-marker end nil)
     (when org-mime--saved-temp-window-config
-      (set-window-configuration org-mime--saved-temp-window-config)
-      (setq org-mime--saved-temp-window-config nil))))
+      (unwind-protect
+          (set-window-configuration org-mime--saved-temp-window-config)
+        (setq org-mime--saved-temp-window-config nil)))
+
+    ;; After everything is cleaned up and the window configuration is
+    ;; restored, kill the edit buffer
+    (kill-buffer edit-buffer)))
 
 (defvar org-mime-src-mode-map
   (let ((map (make-sparse-keymap)))
@@ -848,39 +908,48 @@ Following headline properties can determine the mail headers.
 
 (defun org-mime-edit-mail-in-org-mode ()
   "Call a special editor to edit the mail body in `org-mode'."
-  (interactive)
+  (interactive nil message-mode)
   ;; see `org-src--edit-element'
   (cond
    ((eq major-mode 'org-mode)
     (message "This command is not for `org-mode'."))
    (t
-    (setq org-mime--saved-temp-window-config (current-window-configuration))
-    (let* ((beg (copy-marker (org-mime-mail-body-begin)))
-           (end (copy-marker (or (org-mime-mail-signature-begin) (point-max))))
+    (let* ((window-config (current-window-configuration))
+           (beg (copy-marker (org-mime-mail-body-begin)))
+           (end (copy-marker (or (org-mime-mail-signature-begin) (point-max)) t))
            (bufname "OrgMimeMailBody")
            (buffer (generate-new-buffer bufname))
            (overlay (org-mime-src--make-source-overlay beg end))
            (text (buffer-substring-no-properties beg end)))
 
-      (setq org-mime-src--beg-marker beg)
-      (setq org-mime-src--end-marker end)
-      ;; don't use local-variable because only user can't edit multiple emails
-      ;; or multiple embedded org code in one mail
-      (setq org-mime-src--overlay overlay)
-
-      (save-excursion
-        (delete-other-windows)
-        (org-switch-to-buffer-other-window buffer)
+      ;; Set up BUFFER before displaying it so that, when
+      ;; `org-mime-obey-display-buffer-p' is non-nil,
+      ;; `display-buffer-alist' (which uses `buffer-match-p'
+      ;; internally) can utilize BUFFER's name, major mode, buffer
+      ;; local variables, etc. as matching conditions
+      (with-current-buffer buffer
         (erase-buffer)
-        (insert org-mime-src--hint)
+        (insert org-mime-instructions-hint)
         (insert text)
         (goto-char (point-min))
         (org-mode)
-        (org-mime-src-mode))))))
+        (org-mime-src-mode)
+        ;; Set relevant local variables in `org-mime-src-mode' org
+        ;; buffer
+        (setq org-mime--saved-temp-window-config window-config
+              org-mime-src--beg-marker beg
+              org-mime-src--end-marker end
+              org-mime-src--overlay overlay))
+
+      ;; Display BUFFER
+      (if org-mime-obey-display-buffer-p
+          (pop-to-buffer buffer)
+        (delete-other-windows)
+        (org-switch-to-buffer-other-window buffer))))))
 
 (defun org-mime-revert-to-plain-text-mail ()
   "Revert mail body to plain text."
-  (interactive)
+  (interactive nil message-mode)
   (let* ((txt-sep "<#part type=text/plain>")
          (html-sep "<#part type=text/html>")
          mail-beg
